@@ -5,7 +5,7 @@ use Carp;
 use DBI;
 use base qw(DBI::db);
 
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 
 =head1 NAME
 
@@ -365,43 +365,13 @@ sub _generate_sql {
 
     if ($type eq 'UPDATE' || $type eq 'DELETE' || $type eq 'SELECT' || $type eq 'COUNT')
     {
-        if ($where && !ref $where) {
-            $sql .= " WHERE " . $where;
-        } elsif ( ref $where eq 'HASH' ) {
-            my @stmts;
-            foreach my $k ( sort keys %$where ) {
-                my $v = $where->{$k};
-                if ( ref $v eq 'HASH' ) {
-                    my $not = delete $v->{'not'};
-                    while (my($op,$value) = each %$v ) {
-                        my ($cond, $add_bind_param) 
-                            = $self->_get_where_sql($op, $not, $value);
-                        push @stmts, $self->_quote_identifier($k) . $cond; 
-                        push @bind_params, $v->{$op} if $add_bind_param;
-                    }
-                } else {
-                    my $clause .= $self->_quote_identifier($k);
-                    if ( ! defined $v ) {
-                        $clause .= ' IS NULL';
-                    }
-                    elsif ( ! ref $v ) {
-                        $clause .= '=?';
-                        push @bind_params, $v;
-                    }
-                    elsif ( ref $v eq 'ARRAY' ) {
-                        $clause .= ' IN (' . (join ',', map { '?' } @$v) . ')';
-                        push @bind_params, @$v;
-                    }
-                    push @stmts, $clause;
-                }
-            }
-            $sql .= " WHERE " . join " AND ", @stmts if keys %$where;
-        } elsif (ref $where) {
-            carp "Can't handle ref " . ref $where . " for where";
-            return;
+        if ($where) {
+            my ($where_sql, @where_binds) = $self->generate_where_clauses( $where );
+            return unless $where_sql;   # there was a problem
+            $sql .= " WHERE $where_sql";
+            push(@bind_params, @where_binds);
         }
     }
-
     # Add an ORDER BY clause, if we want to:
     if (exists $opts->{order_by} and defined $opts->{order_by}) {
         $sql .= ' ' . $self->_build_order_by_clause($opts->{order_by});
@@ -438,7 +408,50 @@ sub _generate_sql {
     return ($sql, @bind_params);
 }
 
-sub _get_where_sql {
+sub generate_where_clauses {
+    my ($self, $where) = @_;
+    my $sql = "";
+    my @bind_params;
+    if ($where && !ref $where) {
+        $sql .= $where;
+    } elsif ( ref $where eq 'HASH' ) {
+        my @stmts;
+        foreach my $k ( sort keys %$where ) {
+            my $v = $where->{$k};
+            if ( ref $v eq 'HASH' ) {
+                my $not = delete $v->{'not'};
+                while (my($op,$value) = each %$v ) {
+                    my ($cond, $add_bind_param) 
+                        = $self->_get_where_sql_clause($op, $not, $value);
+                    push @stmts, $self->_quote_identifier($k) . $cond; 
+                    push @bind_params, $v->{$op} if $add_bind_param;
+                }
+            } else {
+                my $clause .= $self->_quote_identifier($k);
+                if ( ! defined $v ) {
+                    $clause .= ' IS NULL';
+                }
+                elsif ( ! ref $v ) {
+                    $clause .= '=?';
+                    push @bind_params, $v;
+                }
+                elsif ( ref $v eq 'ARRAY' ) {
+                    $clause .= ' IN (' . (join ',', map { '?' } @$v) . ')';
+                    push @bind_params, @$v;
+                }
+                push @stmts, $clause;
+            }
+        }
+        $sql .= join " AND ", @stmts if keys %$where;
+    } elsif (ref $where) {
+        carp "Can't handle ref " . ref $where . " for where";
+        return;
+    }
+    return ($sql, @bind_params);
+}
+
+
+sub _get_where_sql_clause {
     my ($self, $op, $not, $value) = @_;
 
     $op = lc $op;
@@ -639,6 +652,46 @@ L<Dancer> and L<Dancer2>
 
 L<DBI>
 
+=head1 LICENSE AND COPYRIGHT
+
+Copyright 2016 David Precious.
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of the the Artistic License (2.0). You may obtain a
+copy of the full license at:
+
+L<http://www.perlfoundation.org/artistic_license_2_0>
+
+Any use, modification, and distribution of the Standard or Modified
+Versions is governed by this Artistic License. By using, modifying or
+distributing the Package, you accept this license. Do not use, modify,
+or distribute the Package, if you do not accept this license.
+
+If your Modified Version has been derived from a Modified Version made
+by someone other than you, you are nevertheless required to ensure that
+your Modified Version complies with the requirements of this license.
+
+This license does not grant you the right to use any trademark, service
+mark, tradename, or logo of the Copyright Holder.
+
+This license includes the non-exclusive, worldwide, free-of-charge
+patent license to make, have made, use, er to sell, sell, import and
+otherwise transfer the Package with respect to any patent claims
+licensable by the Copyright Holder that are necessarily infringed by the
+Package. If you institute patent litigation (including a cross-claim or
+counterclaim) against any party alleging that the Package constitutes
+direct or contributory patent infringement, then this Artistic License
+to you shall terminate on the date that such litigation is filed.
+
+Disclaimer of Warranty: THE PACKAGE IS PROVIDED BY THE COPYRIGHT HOLDER
+AND CONTRIBUTORS "AS IS' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
+THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+PURPOSE, OR NON-INFRINGEMENT ARE DISCLAIMED TO THE EXTENT PERMITTED BY
+YOUR LOCAL LAW. UNLESS REQUIRED BY LAW, NO COPYRIGHT HOLDER OR
+CONTRIBUTOR WILL BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, OR
+CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THE PACKAGE,
+EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 =cut
 
 1;
